scenario = "PCL_lesson3.sce";

# Make a logfile for this
no_logfile = false;

# Some defaults
default_background_color = 128, 128, 128;
default_text_color = 0, 0, 0;
default_font_size = 32;

# One button (1 on the keyboard)
active_buttons = 1;
button_codes = 1; 

# Here is a list of the columns that will show up in the event file.
# There will be a column called "Condition", which consists of a string, and another column called "Word",
# which consists of a string
stimulus_properties = Condition, string, Word, string;

# In the event codes to send to Presentation, the Condition and Word will be separated by a ; (in the actual
#	logfile they'll be tab-delimited, though, FYI)
event_code_delimiter = ";";


# SCE object definitions start here
begin;

### This is an object that presents some text until the correct button is pressed
trial {				#-----------------   INSTRUCTIONS   -----------------          

	# End the event when the correct button is pressed
   trial_type = correct_response;

	# Never time-out
   trial_duration = forever;

	# Define a stimulus event
	stimulus_event {
		# The stimulus is a pic
		picture {              
			# The pic consists of text
			text {
				# Just put a placeholder for "caption" now; the caption will be filled in by the PCL code
				caption = " "; preload=false;
				max_text_width=1250;
			### GIVE THIS TEXT OBJECT A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
			} instruction_caption;
			# Where the picture will appear on the screen
			x = -75; y = 0;
		};
	
		# The button that can end the stimulus event
		target_button = 1;

	### GIVE THE STIMULUS EVENT A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
	} instruction_event;

### GIVE THE TRIAL A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
} instruction_trial; 



### Here's a new SCE object. This one, instead of presenting text until a button press, will
###	present text for a fixed amount of time.
trial {	
	# End the event only after a fixed time
	trial_type = fixed;
	
	# The duration of the trial will be 500 ms (and we can change this in PCL whenever we want!
	trial_duration = 500;

	stimulus_event {
		# The stimulus is a pic
		picture {              
			# The pic consists of text
			text {
				# Just put a placeholder for "caption" now; the caption will be filled in by the PCL code
				caption = " "; preload=false;
				max_text_width=1250;
			### GIVE THIS TEXT OBJECT A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
			} word_caption;
			# Where the picture will appear on the screen
			x = -75; y = 0;
		};

	### GIVE THE STIMULUS EVENT A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
	} word_event;

### GIVE THE TRIAL A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
} word_trial; 


### Here's another SCE object that just presents a fixed ITI
trial {	
	# End the event only after a fixed time
	trial_type = fixed;
	
	# The duration of the trial will be 500 ms (and we can change this in PCL whenever we want!
	trial_duration = 500;

	stimulus_event {
		# The stimulus is a pic
		picture {              
			# The pic consists of text
			text {
				# Since it's an ITI, we can keep this as a blank screen
				caption = " ";
			};
			# Where the picture will appear on the screen
			x = -75; y = 0;
		};

	### GIVE THE STIMULUS EVENT A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
	} ITI_event;

### GIVE THE TRIAL A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
} ITI_trial; 



### PCL code starts here
begin_pcl;

### Let's once again define our favorite subroutine. Remember what this does?
sub
	show_a_message( string message )
begin
	instruction_caption.set_caption( message );
	instruction_caption.redraw();
	instruction_trial.present();
	return;
end;


show_a_message( "Let's return to our previous experiment with \"cat\", \"dog\", \"elephant\", and \"potato\"." );
show_a_message( "Imagine that these words are in two conditions: 'housepet' and 'non-housepet'. Each time a word is presented, " +
						"we want the logfile to show the condition, as well as the word. Let's make a slightly more complicated array " +
						"\(a two-dimensional array\) to hold that information.");
						
### This array has 4 elements (the words of the experiment), and each of those elements has two sub-elements (the word itself,
###	and that word's condition). So stimuli[1][1] tells us the first word ("cat"), and stimuli[1][2] tells us that that word
###	is in the "housepet" condition
array <string> stimuli[4][2] = {
	{"cat", "pet"},
	{"dog", "pet"},
	{"elephant", "nonpet"},
	{"potato", "nonpet"}
};


show_a_message( "As before, let's create an \order\" loop, shuffle it, and then use it to present our stimuli randomly. This time," +
						"rather than using our instructions_trial SCE event \(which presents the words until a button-press\), we'll use " +
						"a different object \(which was defined above in SCE\) which instead presents the trial for a fixed amount of time."); 


array <int> order[0];
loop
	int i=1
until
	i > stimuli.count()
begin
	order.add(i);
	i = i + 1;
end;
order.shuffle();


# present the stims
loop
	int i=1
until
	# We know that the length of the array "order" is also the number of stimuli that we want to present
	i>order.count()
begin
	# Show the first sub-element (the word itself) of the order[i]-th stimulus
	word_caption.set_caption( stimuli[ order[i] ][1] );
	word_caption.redraw();
	word_trial.present();
	
	# After the word, present an ITI. (This SCE object is defined up above.)
	ITI_trial.present();
	
	# Don't forget this!
	i = i + 1;
end;


show_a_message( "Ok, so that presents the stimuli. But we want to keep a log file. To do that, we can use the .set_event_code\(\) " +
						"function for SCE stimulus events." );
						
						
loop
	int i=1
until
	# We know that the length of the array "order" is also the number of stimuli that we want to present
	i>order.count()
begin
	# In a moment what we'll be showing will be the first sub-element (the word itself) of the order[i]-th stimulus
	word_caption.set_caption( stimuli[ order[i] ][1] );
	word_caption.redraw();
	
	# This event code will consist of the condition label, then a semicolon, then the word itself. They'll show up
	#	as separate columns in the log file.
	# Note that, for this to work, you also need to have set up the 'stimulus_properties' and 'event_code_delimiter'
	#	values in the header of the SCE file. Scroll up to the top of the SCE code to see how these look.
	word_event.set_event_code( stimuli[order[i] ][2] + ";" + stimuli[order[i]][1] );
	
	# Actually show the word
	word_trial.present();
	
	# After the word, present an ITI. (This SCE object is defined up above.)
	ITI_trial.present();
	
	# Don't forget this!
	i = i + 1;
end;
						
						
show_a_message( "If you opened the log file after watching that, you will have seen that each stimulus now has a bunch of event " +
						"information spat out with it. How do we get port codes to MEG?");
						
show_a_message( "I can't illustrate how you set up the hardware, without actually being at the MEG computer. But from the PCL standpoint, " +
						"it's quite simple. Just as you can use set_event_code() to determine what goes to the logfile, use set_port_code() " +
						"to determine what does to the MEG! It just needs to be any integer from 1 to 256 (for our machine).\n\n" +
						"That code will be sent to whatever output port was chosen in the Settings window of the Presentation GUI." );
						
show_a_message( "Here we saw examples of modifying properties (the event_code and port_code) of an SCE stimulus_event object. In the first lesson " +
						"we also saw how to modify another property (caption) of an SCE text object. In fact, almost any property of any SCE object can be " +
						"modified like this--if you look in the Presentation documentation you will find built-in functions for modifying SCE objects!" );

show_a_message( "for example, let's modify the \"trial_duration\" property of our words, on the fly! Here, each word will be presented for a random amount of time instead of 500 ms." );

loop
	int i=1
until
	i>order.count()
begin
	# Set the word to be displayed
	word_caption.set_caption( stimuli[ order[i] ][1] );
	word_caption.redraw();
	
	# Set the duration to a random number (using the built-in subroutine called "random"!) between 100
	#	and 3000 ms
	word_trial.set_duration( random(100, 3000) );
	
	# actually present the word
	word_trial.present();

	# actually present the ITI
	ITI_trial.present();

	# Don't forget this!
	i = i + 1;
end;

show_a_message( "With the ability to modify SCE objects' properties on the fly, to store lots of stimulus-related information in arrays, and " +
						"to have PCL make decisions via conditionals and execute many operations quickly using loops and subroutines, you can now " +
						"program pretty much any experiment!" );