scenario = "PCL_lesson3.sce";

# Don't make a logfile for this
no_logfile = true;

# Some defaults
default_background_color = 128, 128, 128;
default_text_color = 0, 0, 0;
default_font_size = 32;

# One button (1 on the keyboard)
active_buttons = 1;
button_codes = 1; 


# SCE object definitions start here
begin;


### This is an object that presents some text until the correct button is pressed
trial {				#-----------------   INSTRUCTIONS   -----------------          

	# End the event when the correct button is pressed
   trial_type = correct_response;

	# Never time-out
   trial_duration = forever;

	# Define a stimulus event
	stimulus_event {
		# The stimulus is a pic
		picture {              
			# The pic consists of text
			text {
				# Just put a placeholder for "caption" now; the caption will be filled in by the PCL code
				caption = " "; preload=false;
				max_text_width=1250;
			### GIVE THIS TEXT OBJECT A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
			} instruction_caption;
			# Where the picture will appear on the screen
			x = -75; y = 0;
		};
	
		# The button that can end the stimulus event
		target_button = 1;

	### GIVE THE STIMULUS EVENT A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
	} instruction_event;

### GIVE THE TRIAL A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
} instruction_trial; 




### PCL code starts here
begin_pcl;

### Let's once again define our favorite subroutine. Remember what this does?
sub
	show_a_message( string message )
begin
	instruction_caption.set_caption( message );
	instruction_caption.redraw();
	instruction_trial.present();
	return;
end;


show_a_message( "Let's say you're doing a lexical decision experiment and you have four stimuli: \"cat\", " +
						"\"dog\", \"elephant\", and \"potato\". You want to present all these stimuli. If you want to " +
						"make your life hard, you could use something like:\n" +
						"show_a_message\(\"cat\"\);\n" +
						"show_a_message\(\"dog\"\);\n" +
						"show_a_message\(\"elephant\"\);\n" +
						"show_a_message\(\"potato\"\);\n" );
show_a_message( "But imagine how much of a pain it would be to program PCL that way if you had 1000 stimuli! And " +
						"what if you want to present them in a random order? You'd have to paste those in all over again!" );
show_a_message( "Here, a new PCL data structure, called the ARRAY, can come to our rescue. An array is a list of objects " +
						"of the same type \(for example, a list of strings--maybe the list of strings that are your stimuli\).");
show_a_message( "Once you've put your stimuli into an array, you can loop through the array and present them all. Here's an example " +
						"\(as usual, open the PCL file to see the code behind this\):" );
						
### Create an array. The part in <> tells us the items in the array are each strings. Then we see that the array
###	is called "stimuli", and it's going to be 4 items long
array <string> stimuli[4] = {
	# Now have a comma-separated list of the things in the array
	"cat", "dog", "elephant", "potato"
# Now we've created the array
};

# We can refer to elements of the array by indexing. For example, stimuli[1] gives us the first item of the array;
#	stimuli[3] gives us the third, etc. So now you see how we can loop through and present all the stimuli in the array:

loop
	# as usual, we start the loop at 1
	int i=1
until
	# .count() is a built-in subroutine that returns the number of items in an array. So this tells us to keep
	#	looping until we've done as many iterations as there are stimuli in this array.
	i > stimuli.count()
begin
	# show the i-th word in the array!
	show_a_message( stimuli[i] );
	
	# Don't forget this!
	i = i + 1;
end;





show_a_message( "Now let's say you want to show these in a random order. There is a built-in function called .shuffle\(\) " +
						"which rearranges the elements of an array in a random order. So before running the loop above, we could have " +
						"run stimuli.shuffle\(\) to put the words in a random order." );
show_a_message( "But let's go one step beyond that. Sometimes you want to present things in a random order but still keep the original " +
						"array in its original order \(for example, maybe in addition to that array, you have a separate array of the word frequencies, " +
						"which you want for looking at lexical decision RTs; obviously, if you shuffle\(\) both of these arrays, you will lose track of " +
						"which frequency goes with which word!" );
show_a_message( "So here's a trick for how to present the items of an array in a random order, WITHOUT directly shuffling the array. " +
						"In the process, we'll also see some useful funtions for working with arrays. \(As usual, open the PCL code to see how." );
						
						

### Ok, here's the general strategy:
###	We have an array, "stimuli", which is 4 items long. We want to present them in a random order without shuffling the array
###	Each item in "stimuli" has an index: there is a 1st item, a 2nd item, a 3rd item, and a 4th item
###	So we'll create a new array, called "order", which will have 4 numbers (1 through 4) because "stimuli" has 4 items
###	Then we'll shuffle that array, and go through it in order
###	For example, after shuffling, the "order" array might look like: {3, 2, 4, 1}
###	When we go through that in order, we will first present stimuli[3], then stimuli[2], then stimuli[4], then stimuli[1]. So it's random
###	Got it? Now let's do it!

# Create the array. This time, it's an array of integers, rather than an array of strings. And it's called "order"
#	Notice that, instead of putting in the actual numbers (like we put in the actual strings when we created the "stimuli" array),
#	this time we are making it start as an empty array: it has length [0] and there are actually no elements in it. Then
#	we're going to use a PCL loop to fill it in with numbers.
#	(note that we could have just done: array <int> order[4] = {1, 2, 3, 4};
#	(if we did that, the array would already be created. But wouldn't that be a pain to type if we had 1000 stimuli instead of 4?
#	(so here we'll look at how to create an array like this more flexibly, using a PCL loop
array <int> order[0];

# Now we'll loop through, from 1 until however many stimuli there are in "stimuli" (it happens to be 4). Each time, we'll add that
#	number to "order"
loop
	int i=1
until
	i > stimuli.count()
begin
	
	# Use the built-in function .add() to add the value if i (1 the first time, 2 the second time, etc.) onto our "order" array
	order.add(i);
	
	# Don't forget this!
	i = i + 1;
end;
# Now that the loop is done, "order" will have the values: {1, 2, 3, 4}

# now put the trials in a random order:
order.shuffle();


# Now let's go ahead and present each of the stimuli, but this time in a random order!
loop
	int i=1
until
	# We know that the length of the array "order" is also the number of stimuli that we want to present
	i>order.count()
begin
	# Show a message. Instead of showing the i-th (e.g. 1st, 2nd, 3rd, 4th) stimulus, we will find the
	#	i-th number of "order" (remember, we shuffled it, so the 1st number in "order" might be 1, or 2, or 3, or 4),
	#	then we'll take that number and present the n-th (that number) item in "stimuli"
	show_a_message( stimuli[ order[i] ] );
	
	# Don't forget this!
	i = i + 1;
end;


show_a_message( "Ok, that PCL code may have looked pretty complicated. But it's an extremely useful trick for presenting stimuli. " +
						"Plus, now that you know  how to do that, you're an expert on arrays--that code used many of the most common array functions!" );
						
						
show_a_message( "So now you can present all kinds of stimuli! But you want to actually send triggers to the MEG, right? Our last lesson will " +
						"look at that, and in the process we'll also learn more about how to manipulate SCE objects in many ways." );
						
