scenario = "PCL_lesson3.sce";

# Don't make a logfile for this
no_logfile = true;

# Some defaults
default_background_color = 128, 128, 128;
default_text_color = 0, 0, 0;
default_font_size = 32;

# One button (1 on the keyboard)
active_buttons = 1;
button_codes = 1; 


# SCE object definitions start here
begin;


### This is an object that presents some text until the correct button is pressed
trial {				#-----------------   INSTRUCTIONS   -----------------          

	# End the event when the correct button is pressed
   trial_type = correct_response;

	# Never time-out
   trial_duration = forever;

	# Define a stimulus event
	stimulus_event {
		# The stimulus is a pic
		picture {              
			# The pic consists of text
			text {
				# Just put a placeholder for "caption" now; the caption will be filled in by the PCL code
				caption = " "; preload=false;
				max_text_width=1250;
			### GIVE THIS TEXT OBJECT A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
			} instruction_caption;
			# Where the picture will appear on the screen
			x = -75; y = 0;
		};
	
		# The button that can end the stimulus event
		target_button = 1;

	### GIVE THE STIMULUS EVENT A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
	} instruction_event;

### GIVE THE TRIAL A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
} instruction_trial; 




### PCL code starts here
begin_pcl;

### Let's once again define our favorite subroutine. Remember what this does?
sub
	show_a_message( string message )
begin
	instruction_caption.set_caption( message );
	instruction_caption.redraw();
	instruction_trial.present();
	return;
end;


show_a_message( "Any human can tell a computer what to do. For example, if we wanted to present \"hello\" on the screen " +
						"500 times, we could repeat show_a_message\(\"hello\"\) 500 times in PCL. But that would be a pain to type." );
show_a_message( "The benefit of programming languages is that they can do what is called PROGRAM CONTROL FLOW: they can do repetitive " +
						"tasks like that much faster than a human can, provided you give them some simple instructions telling them  how to do it." );
show_a_message( "For example, rather than you calling the function 500 times, you can do this via a LOOP that tells PCL to " +
						"execute some chunk of code 500 times. Here's an example \(for brevity's sake, we'll just do it 5 times \); " +
						"you can open the PCL file itself to see how this loop worked! For now, here goes:");

### Tell PCL we're going to start a loop
loop
	# The loop needs a way of keeping track of where it is. For this, we'll create a counter variable, called 'i',
	#	which keeps track of how many times the loop has run. It begins at 1 (because we said so below). Every time
	#	the loop finishes one iteration, it'll go up by one. Note that there's no semicolon here.
	int i=1
until
	# We'll stop the loop when i passes 5. (since i started at 1, this means the loop will run 5 times.
	i > 5

# tell the loop to start!
begin
	# We want to present this message each time the loop runs
	show_a_message( "hello" );

	# Now that the loop has run once, increment the counter by 1.
	#	(Don't forget this part! If you do, the loop will run forever, becauase i will never pass 5--you'll be trapped in PCL!)
	i = i + 1;
# end the loop
end;



show_a_message( "That was cool, right! You can also make a loop do different things each time; for example, here's one that " +
					"also runs 5 times, but instead of presenting the same text each time, it presents a number saying which repetition " +
					"it is on." );


### Here's a new loop
loop
	int i = 1
until
	i > 5
begin
	# This time, instead of showing "hello" every time, we'll show the value of i.
	#	Since i is changing as the loop runs, the loop will show a different message on each iteration!
	show_a_message( string(i) );
	
	# Don't forget this!
	i = i + 1;
end;



show_a_message( "What's more, PCL can make decisions. This is done using IF statements. If some condition is met, PCL will run some code; " +
						"otherwise it won't. Let's look at an example in the form of a loop that only prints odd numbers. \( To make that work, " +
						"we'll also need a subroutine that figures out if a number is even or odd, so you can check the PCL code here to see an " +
						"example of doing math in PCL.\)" );
						
						
### Here's a subroutine to figure out if a number is odd
sub
	# This subroutine returns a data type we haven't seen before: a "bool". Short for "boolean",
	#	this is a value of either "true" or "false".
	# The subroutine is called "is_odd" and its input (which will be referred to as "x" within
	#	the subroutine) is a single integer
	bool is_odd( int x )
begin
	# Let's start out by assuming x is odd (so the result to be returned will be "true"), and we'll change
	#	this to "false" if x turns out to be even. (This isn't really important, it could have been done
	#	the other way around if we preferred.)
	bool result = true;
	
	# The % here is modulus division. It gives you the remainder of x / 2. Of course, if the
	# remainder is 0, then x is an even number.
	if x % 2 == 0 then
		# If x was even, then the result of is_odd() should be "false"
		result = false;
	end;

	# Send the result (true or false) back to PCL
	return result;
end;

### Now that we have our subroutine, let's use it in a loop.

loop
	int i=1
until
	i>5
begin
	# if the result of is_odd(i) is "true" (i.e., if i is an odd number), then we'll execute the
	#	lines of code that come after "then". If not, we won't do those
	if is_odd(i) then
		show_a_message( string(i) );
	end;
	
	# Never forget this!
	i = i + 1;
end;




show_a_message( "Ok, so we've just seen an example of PCL making decisions: execute some code if a condition is meant, otherwise do not. " +
						"PCL can also do different things if the condition is not met. Between IF and END, you can also have \"elseif\" " +
						"\(which means, if the first condition was not met, then check to see if some second condition is met, and if it is met then " +
						"do the things for that condition\) and \"else\" \(which means, if no other conditions are met, then eventually do this\). " +
						"Let's look at an example." );
						


loop 
	int i=1
until
	i>5
begin
	
	# If i is 3 on this iteration, then do the thing on the next line
	if i==3 then
		show_a_message( "Hey, i is 3 this time!" );
	
	# Otherwise, if i is 4 on this iteration, then do the thing on the next line
	elseif i==4 then
		show_a_message( "i isn't 3 this time, but i is 4 this time!" );
	
	# Otherwise, if i is 1 on this iteration, then do the thing on the next line
	elseif i==1 then
		show_a_message( "i isn't 3 this time, but i is 1 this time!" );
		
	# If none of the previous things got triggered (i.e., if i is neither 3, 4, nor 1), then do this by default.
	else
		show_a_message( "i is neither 3 nor 4 nor 1, and I haven't told PCL to do anything special in this situation." );
		
	# now the conditional is done
	end;

	# Don't forget this!!!
	i = i + 1;
end;


show_a_message( "Ok, so now you know how to make PCL repeat things many times, and how to make PCL make decisions. " +
						"\(You can read the Presentation documentation for a lot more information on how conditionals work, " +
						"for example how to include AND or OR in your conditionals, and more details about how, logically speaking, " +
						"PCL evaluates the \"truth\" of the conditional.\)" );
show_a_message( "But for your experiment, you probably don't want to just present some boring numbers in order. You probably have " +
						"stimuli that you care about, and want to present them. In the next lesson we'll look at ARRAYs, which are " +
						"PCL's way of doing that." );