scenario = "PCL_lesson2.sce";

# Don't make a logfile for this
no_logfile = true;

# Some defaults
default_background_color = 128, 128, 128;
default_text_color = 0, 0, 0;
default_font_size = 32;

# One button (1 on the keyboard)
active_buttons = 1;
button_codes = 1; 


# SCE object definitions start here
begin;


### This is an object that presents some text until the correct button is pressed
trial {				#-----------------   INSTRUCTIONS   -----------------          

	# End the event when the correct button is pressed
   trial_type = correct_response;

	# Never time-out
   trial_duration = forever;

	# Define a stimulus event
	stimulus_event {
		# The stimulus is a pic
		picture {              
			# The pic consists of text
			text {
				# Just put a placeholder for "caption" now; the caption will be filled in by the PCL code
				caption = " "; preload=false;
				max_text_width=800;
			### GIVE THIS TEXT OBJECT A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
			} instruction_caption;
			# Where the picture will appear on the screen
			x = -75; y = 0;
		};
	
		# The button that can end the stimulus event
		target_button = 1;

	### GIVE THE STIMULUS EVENT A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
	} instruction_event;

### GIVE THE TRIAL A NAME, SO IT CAN BE MANIPULATED LATER ON IN PCL
} instruction_trial; 




### PCL code starts here
begin_pcl;

### The next bit of code is called a SUBROUTINE. It defines several lines of code that,
###	any time you call "show_a_message()", will get run the same way. Most importantly,
###	you can call show_a_message() with some text in it (e.g., show_a_message("Hello world!")), and the
###	text in the parentheses will get "plugged in" to the code that's getting run. This
###	is a good way to make your life easier: if you have some code that you run over and
###	over again, like for redrawing text and then presenting it, then instead of having
###	a ton of copies of that code (and having to change them all if you make changes to the
###	script), you just have one copy--inside of this subroutine--that gets called over and
###	over again.

# begin the subroutine
sub
	# The name of the subroutine is show_a_message. It takes one input parameter, which is
	#	a string. The string that you feed to the subroutine will be called "message" inside
	#	the subroutine
	show_a_message( string message )
# Starting here, we have the code that will actually get run every time you call the subroutine.
begin
	
	# Use the set_caption() method of the SCE object instruction_caption, to change the text that
	#	that will be displayed. The text to put in will be whatever "message" was when you called
	#	the subroutine.
	instruction_caption.set_caption( message );
	# Redraw the picture that will be presented on screen (this needs to be done every time you change a SCE picture object)
	instruction_caption.redraw();
	# Actually present the instructions
	instruction_trial.present();
	# This lets Presentation know that the subroutine is over.
	return;
end;



### Now that the subroutine has been defined, we can actually use it in PCL code:
show_a_message( "Hello! This text is being presented using a subroutine! (Press 1 to keep reading.)" );
show_a_message( "And now I'm presenting some different text!" );
show_a_message( "This is much easier, right?" );





### Subroutines don't just need to do code; they can also spit back out some value, which you
###	can use in PCL later. For example, let's create a subroutine that takes a number
###	you give to it, adds 5 to that number, and returns the result:

sub
	# Unlike the previous subroutine, this one has "int" at the beginning, which tells Presentation
	#	that the value returned by the subroutine will be an integer.
	#	Then we see that the name of the subroutine is "add_five". It takes an integer as an input,
	#	and that integer will be called "x" within the subroutine
	int add_five( int x )
begin
	# Add 5 to whatever number the subroutine was given.
	int sum = x + 5;
	# Return that sum back to PCL!
	return sum;
end;

### Let's show this in action
# we need to wrap the 5 up in a string() subroutine (this is actually a PCL built-in subroutine!) so that it
#	can be combined with the string "Here's a number: " in order to be presented as text on the screen
show_a_message( "Here's a number: " + string(3) );
show_a_message( "Here's what we get when we put that number through the subroutine: " + string( add_five( 3 ) ) );



### Subroutines can have more than one input value, too

sub
	# This is like the previous subroutine, except instead of adding 5 to "x", we add "y" to "x". The
	#	user (well, the PCL code that calls this subroutine) gets to decide what both "x" and "y" are!
	int add_two_numbers( int x, int y )
begin
	# Add y to x.
	int sum = x + y;
	# Return that sum back to PCL!
	return sum;
end;


show_a_message( "Let's try that subroutine with two numbers: " + string( 7 ) + " and " + string( 9 ) );
show_a_message( "Here's the result: " + string( add_two_numbers(7,9) ) );

show_a_message( "Of course, these subroutines are pretty boring. But subroutines can do much more. For example, you can create " +
						"a subroutine called \"present_trial\" that does everything required for presenting one trial in your experiment " +
						"(e.g., figuring out which color picture to load, telling Presentation what trigger to send to the MEG, etc.); " +
						"then, making a whole experiment is just a matter of making a loop that repeats that same subroutine many times. " +
						"That's a lot easier than hard-coding every trial of the experiment in Excel and then pasting it into an SCE template, right?!" );
						
show_a_message( "In the next lesson we'll take a look at the two things that make PCL (and programming in general) so powerful: loops and conditionals." );